import argparse
import os
from typing import Tuple

from pypanther import display
from pypanther.get import get_panther_rules
from pypanther.import_main import NoMainModuleError, import_main
from pypanther.registry import registered_rules


def run(args: argparse.Namespace) -> Tuple[int, str]:
    rules = set()

    if len(args.attributes) > 1 and display.ALL_TABLE_ATTR in args.attributes:
        return 1, f"Cannot use any other attributes with '{display.ALL_TABLE_ATTR}'."

    if display.ALL_TABLE_ATTR in args.attributes:
        args.attributes = display.VALID_RULE_TABLE_ATTRS

    if not args.managed:
        try:
            import_main(os.getcwd(), "main")
        except NoMainModuleError:
            return 1, "No main.py found. Cannot list registered rules without main.py."

    if args.managed is None:
        # If managed is not specified, we will list all rules
        rules = registered_rules(
            log_types=getattr(args, "log_types", None),
            id=getattr(args, "id", None),
            create_alert=getattr(args, "create_alert", None),
            dedup_period_minutes=getattr(args, "dedup_period_minutes", None),
            display_name=getattr(args, "display_name", None),
            enabled=getattr(args, "enabled", None),
            summary_attributes=getattr(args, "summary_attributes", None),
            threshold=getattr(args, "threshold", None),
            tags=getattr(args, "tags", None),
            default_severity=getattr(args, "default_severity", None),
            default_description=getattr(args, "default_description", None),
            default_reference=getattr(args, "default_reference", None),
            default_runbook=getattr(args, "default_runbook", None),
            default_destinations=getattr(args, "default_destinations", None),
        )
    elif args.managed:
        # if managed is True, we will list only Panther managed rules
        rules = set(
            get_panther_rules(
                log_types=getattr(args, "log_types", None),
                id=getattr(args, "id", None),
                create_alert=getattr(args, "create_alert", None),
                dedup_period_minutes=getattr(args, "dedup_period_minutes", None),
                display_name=getattr(args, "display_name", None),
                enabled=getattr(args, "enabled", None),
                summary_attributes=getattr(args, "summary_attributes", None),
                threshold=getattr(args, "threshold", None),
                tags=getattr(args, "tags", None),
                default_severity=getattr(args, "default_severity", None),
                default_description=getattr(args, "default_description", None),
                default_reference=getattr(args, "default_reference", None),
                default_runbook=getattr(args, "default_runbook", None),
                default_destinations=getattr(args, "default_destinations", None),
            ),
        )
    else:
        # if managed is False, we will list only non-Panther managed rules
        panther_rules = get_panther_rules(
            log_types=getattr(args, "log_types", None),
            id=getattr(args, "id", None),
            create_alert=getattr(args, "create_alert", None),
            dedup_period_minutes=getattr(args, "dedup_period_minutes", None),
            display_name=getattr(args, "display_name", None),
            enabled=getattr(args, "enabled", None),
            summary_attributes=getattr(args, "summary_attributes", None),
            threshold=getattr(args, "threshold", None),
            tags=getattr(args, "tags", None),
            default_severity=getattr(args, "default_severity", None),
            default_description=getattr(args, "default_description", None),
            default_reference=getattr(args, "default_reference", None),
            default_runbook=getattr(args, "default_runbook", None),
            default_destinations=getattr(args, "default_destinations", None),
        )
        all_rules = registered_rules(
            log_types=getattr(args, "log_types", None),
            id=getattr(args, "id", None),
            create_alert=getattr(args, "create_alert", None),
            dedup_period_minutes=getattr(args, "dedup_period_minutes", None),
            display_name=getattr(args, "display_name", None),
            enabled=getattr(args, "enabled", None),
            summary_attributes=getattr(args, "summary_attributes", None),
            threshold=getattr(args, "threshold", None),
            tags=getattr(args, "tags", None),
            default_severity=getattr(args, "default_severity", None),
            default_description=getattr(args, "default_description", None),
            default_reference=getattr(args, "default_reference", None),
            default_runbook=getattr(args, "default_runbook", None),
            default_destinations=getattr(args, "default_destinations", None),
        )
        # Get the difference between all rules and panther managed rules
        rules = all_rules - set(panther_rules)
    try:
        match args.output:
            case "text":
                display.print_rule_table(list(rules), attributes=args.attributes, sort_by=args.sort_by)
            case "json":
                display.print_rules_as_json(list(rules), attributes=args.attributes, sort_by=args.sort_by)
            case "csv":
                display.print_rules_as_csv(list(rules), attributes=args.attributes, sort_by=args.sort_by)
            case _:
                return 1, f"Unsupported output: {args.output}"
    except AttributeError as err:
        return 1, f"Invalid attribute was given in --attributes option: {err!s}"

    return 0, ""



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.crowdstrike_fdr import crowdstrike_process_alert_context


@panther_managed
class CrowdstrikeMacosAddTrustedCert(Rule):
    display_name = "CrowdStrike MacOS Added Trusted Cert"
    default_description = "Detects attempt to install a root certificate on MacOS"
    id = "Crowdstrike.Macos.Add.Trusted.Cert-prototype"
    default_reference = "https://docs.panther.com/data-onboarding/supported-logs/crowdstrike#crowdstrike.processrollup2"
    default_severity = Severity.MEDIUM
    log_types = [LogType.CROWDSTRIKE_FDR_EVENT]

    def rule(self, event):
        event_platform = event.get("event_platform", "<UNKNOWN_PLATFORM>")
        fdr_event_type = event.get("fdr_event_type", "<UNKNOWN_FDR_EVENT_TYPE>")
        image_filename = event.deep_get("event", "ImageFileName", default="<UNKNOWN_IMAGE_FILE_NAME>")
        command_line = event.deep_get("event", "CommandLine", default="<UNKNOWN_COMMAND_LINE>")
        return all(
            [
                event_platform == "Mac",
                fdr_event_type == "ProcessRollup2",
                image_filename == "/usr/bin/security",
                "add-trusted-cert" in command_line,
            ],
        )

    def title(self, event):
        aid = event.get("aid", "<UNKNOWN_AID>")
        return f"Crowdstrike: New trusted cert added on device [{aid}]"

    def alert_context(self, event):
        return crowdstrike_process_alert_context(event)

    tests = [
        RuleTest(
            name="Added root cert",
            expected_result=True,
            log={
                "aid": "1234abcdefghijklmnop",
                "aip": "1.2.3.4",
                "cid": "abcde098654321xyz",
                "configbuild": "1007.4.0016804.11",
                "configstatehash": "1260279761",
                "entitlements": "15",
                "event": {
                    "CodeSigningFlags": "570506001",
                    "CommandLine": "security add-trusted-cert -d -r trusted",
                    "ConfigBuild": "1007.4.0016804.11",
                    "ConfigStateHash": "1260279761",
                    "EffectiveTransmissionClass": "2",
                    "Entitlements": "15",
                    "GID": "20",
                    "ImageFileName": "/usr/bin/security",
                    "MD5HashData": "5a29b1ee210395604829dafd744a8039",
                    "MachOSubType": "1",
                    "ParentBaseFileName": "zsh",
                    "ParentProcessId": "488592041865603507",
                    "ProcessEndTime": "",
                    "ProcessGroupId": "488638905187012012",
                    "ProcessStartTime": "1685123957.068",
                    "RGID": "501",
                    "RUID": "501",
                    "RawProcessId": "51673",
                    "SHA1HashData": "0000000000000000000000000000000000000000",
                    "SHA256HashData": "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de",
                    "SVGID": "20",
                    "SVUID": "501",
                    "SessionProcessId": "488592041748162994",
                    "SigningId": "com.apple.Foundation.plutil",
                    "SourceProcessId": "488592041865603507",
                    "SourceThreadId": "0",
                    "Tags": "316",
                    "TargetProcessId": "488638905187012012",
                    "TeamId": "-",
                    "UID": "501",
                    "aid": "1234abcdefghijklmnop",
                    "aip": "1.2.3.4",
                    "cid": "abcde098654321xyz",
                    "event_platform": "Mac",
                    "event_simpleName": "ProcessRollup2",
                    "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                    "name": "ProcessRollup2MacV9",
                    "timestamp": "1685123957235",
                },
                "event_platform": "Mac",
                "event_simplename": "ProcessRollup2",
                "fdr_event_type": "ProcessRollup2",
                "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                "name": "ProcessRollup2MacV9",
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_any_md5_hashes": ["1234abcdefghijklmnop", "5a29b1ee210395604829dafd744a8039", "abcde098654321xyz"],
                "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
                "p_any_sha256_hashes": ["fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de"],
                "p_any_trace_ids": ["1234abcdefghijklmnop", "abcde098654321xyz"],
                "p_event_time": "2023-05-26 17:59:17.235",
                "p_log_type": "Crowdstrike.FDREvent",
                "p_parse_time": "2023-05-26 18:11:03.404",
                "p_row_id": "d6ff1555b3f6e785b3e2bbb218abf524",
                "p_schema_version": 0,
                "p_source_id": "e4962678-a986-44af-a757-18163a2bc963",
                "p_source_label": "Crowdstrike",
                "timestamp": "2023-05-26 17:59:17.235",
            },
        ),
        RuleTest(
            name="Windows log",
            expected_result=False,
            log={
                "aid": "1234abcdefghijklmnop",
                "aip": "1.2.3.4",
                "cid": "abcde098654321xyz",
                "configbuild": "1007.4.0016804.11",
                "configstatehash": "1260279761",
                "entitlements": "15",
                "event": {
                    "CodeSigningFlags": "570506001",
                    "CommandLine": "security add-trusted-cert -d -r trusted",
                    "ConfigBuild": "1007.4.0016804.11",
                    "ConfigStateHash": "1260279761",
                    "EffectiveTransmissionClass": "2",
                    "Entitlements": "15",
                    "GID": "20",
                    "ImageFileName": "/usr/bin/security",
                    "MD5HashData": "5a29b1ee210395604829dafd744a8039",
                    "MachOSubType": "1",
                    "ParentBaseFileName": "zsh",
                    "ParentProcessId": "488592041865603507",
                    "ProcessEndTime": "",
                    "ProcessGroupId": "488638905187012012",
                    "ProcessStartTime": "1685123957.068",
                    "RGID": "501",
                    "RUID": "501",
                    "RawProcessId": "51673",
                    "SHA1HashData": "0000000000000000000000000000000000000000",
                    "SHA256HashData": "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de",
                    "SVGID": "20",
                    "SVUID": "501",
                    "SessionProcessId": "488592041748162994",
                    "SigningId": "com.apple.Foundation.plutil",
                    "SourceProcessId": "488592041865603507",
                    "SourceThreadId": "0",
                    "Tags": "316",
                    "TargetProcessId": "488638905187012012",
                    "TeamId": "-",
                    "UID": "501",
                    "aid": "1234abcdefghijklmnop",
                    "aip": "1.2.3.4",
                    "cid": "abcde098654321xyz",
                    "event_platform": "Mac",
                    "event_simpleName": "ProcessRollup2",
                    "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                    "name": "ProcessRollup2MacV9",
                    "timestamp": "1685123957235",
                },
                "event_platform": "Win",
                "event_simplename": "ProcessRollup2",
                "fdr_event_type": "ProcessRollup2",
                "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                "name": "ProcessRollup2MacV9",
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_any_md5_hashes": ["1234abcdefghijklmnop", "5a29b1ee210395604829dafd744a8039", "abcde098654321xyz"],
                "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
                "p_any_sha256_hashes": ["fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de"],
                "p_any_trace_ids": ["1234abcdefghijklmnop", "abcde098654321xyz"],
                "p_event_time": "2023-05-26 17:59:17.235",
                "p_log_type": "Crowdstrike.FDREvent",
                "p_parse_time": "2023-05-26 18:11:03.404",
                "p_row_id": "d6ff1555b3f6e785b3e2bbb218abf524",
                "p_schema_version": 0,
                "p_source_id": "e4962678-a986-44af-a757-18163a2bc963",
                "p_source_label": "Crowdstrike",
                "timestamp": "2023-05-26 17:59:17.235",
            },
        ),
        RuleTest(
            name="Wrong FDR event type",
            expected_result=False,
            log={
                "aid": "1234abcdefghijklmnop",
                "aip": "1.2.3.4",
                "cid": "abcde098654321xyz",
                "configbuild": "1007.4.0016804.11",
                "configstatehash": "1260279761",
                "entitlements": "15",
                "event": {
                    "CodeSigningFlags": "570506001",
                    "CommandLine": "security add-trusted-cert -d -r trusted",
                    "ConfigBuild": "1007.4.0016804.11",
                    "ConfigStateHash": "1260279761",
                    "EffectiveTransmissionClass": "2",
                    "Entitlements": "15",
                    "GID": "20",
                    "ImageFileName": "/usr/bin/security",
                    "MD5HashData": "5a29b1ee210395604829dafd744a8039",
                    "MachOSubType": "1",
                    "ParentBaseFileName": "zsh",
                    "ParentProcessId": "488592041865603507",
                    "ProcessEndTime": "",
                    "ProcessGroupId": "488638905187012012",
                    "ProcessStartTime": "1685123957.068",
                    "RGID": "501",
                    "RUID": "501",
                    "RawProcessId": "51673",
                    "SHA1HashData": "0000000000000000000000000000000000000000",
                    "SHA256HashData": "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de",
                    "SVGID": "20",
                    "SVUID": "501",
                    "SessionProcessId": "488592041748162994",
                    "SigningId": "com.apple.Foundation.plutil",
                    "SourceProcessId": "488592041865603507",
                    "SourceThreadId": "0",
                    "Tags": "316",
                    "TargetProcessId": "488638905187012012",
                    "TeamId": "-",
                    "UID": "501",
                    "aid": "1234abcdefghijklmnop",
                    "aip": "1.2.3.4",
                    "cid": "abcde098654321xyz",
                    "event_platform": "Mac",
                    "event_simpleName": "ProcessRollup2",
                    "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                    "name": "ProcessRollup2MacV9",
                    "timestamp": "1685123957235",
                },
                "event_platform": "Mac",
                "event_simplename": "DnsRequest",
                "fdr_event_type": "DnsRequest",
                "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                "name": "ProcessRollup2MacV9",
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_any_md5_hashes": ["1234abcdefghijklmnop", "5a29b1ee210395604829dafd744a8039", "abcde098654321xyz"],
                "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
                "p_any_sha256_hashes": ["fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de"],
                "p_any_trace_ids": ["1234abcdefghijklmnop", "abcde098654321xyz"],
                "p_event_time": "2023-05-26 17:59:17.235",
                "p_log_type": "Crowdstrike.FDREvent",
                "p_parse_time": "2023-05-26 18:11:03.404",
                "p_row_id": "d6ff1555b3f6e785b3e2bbb218abf524",
                "p_schema_version": 0,
                "p_source_id": "e4962678-a986-44af-a757-18163a2bc963",
                "p_source_label": "Crowdstrike",
                "timestamp": "2023-05-26 17:59:17.235",
            },
        ),
        RuleTest(
            name="Wrong executable path",
            expected_result=False,
            log={
                "aid": "1234abcdefghijklmnop",
                "aip": "1.2.3.4",
                "cid": "abcde098654321xyz",
                "configbuild": "1007.4.0016804.11",
                "configstatehash": "1260279761",
                "entitlements": "15",
                "event": {
                    "CodeSigningFlags": "570506001",
                    "CommandLine": "security add-trusted-cert -d -r trusted",
                    "ConfigBuild": "1007.4.0016804.11",
                    "ConfigStateHash": "1260279761",
                    "EffectiveTransmissionClass": "2",
                    "Entitlements": "15",
                    "GID": "20",
                    "ImageFileName": "/home/test/bin/security",
                    "MD5HashData": "5a29b1ee210395604829dafd744a8039",
                    "MachOSubType": "1",
                    "ParentBaseFileName": "zsh",
                    "ParentProcessId": "488592041865603507",
                    "ProcessEndTime": "",
                    "ProcessGroupId": "488638905187012012",
                    "ProcessStartTime": "1685123957.068",
                    "RGID": "501",
                    "RUID": "501",
                    "RawProcessId": "51673",
                    "SHA1HashData": "0000000000000000000000000000000000000000",
                    "SHA256HashData": "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de",
                    "SVGID": "20",
                    "SVUID": "501",
                    "SessionProcessId": "488592041748162994",
                    "SigningId": "com.apple.Foundation.plutil",
                    "SourceProcessId": "488592041865603507",
                    "SourceThreadId": "0",
                    "Tags": "316",
                    "TargetProcessId": "488638905187012012",
                    "TeamId": "-",
                    "UID": "501",
                    "aid": "1234abcdefghijklmnop",
                    "aip": "1.2.3.4",
                    "cid": "abcde098654321xyz",
                    "event_platform": "Mac",
                    "event_simpleName": "ProcessRollup2",
                    "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                    "name": "ProcessRollup2MacV9",
                    "timestamp": "1685123957235",
                },
                "event_platform": "Mac",
                "event_simplename": "ProcessRollup2",
                "fdr_event_type": "ProcessRollup2",
                "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                "name": "ProcessRollup2MacV9",
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_any_md5_hashes": ["1234abcdefghijklmnop", "5a29b1ee210395604829dafd744a8039", "abcde098654321xyz"],
                "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
                "p_any_sha256_hashes": ["fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de"],
                "p_any_trace_ids": ["1234abcdefghijklmnop", "abcde098654321xyz"],
                "p_event_time": "2023-05-26 17:59:17.235",
                "p_log_type": "Crowdstrike.FDREvent",
                "p_parse_time": "2023-05-26 18:11:03.404",
                "p_row_id": "d6ff1555b3f6e785b3e2bbb218abf524",
                "p_schema_version": 0,
                "p_source_id": "e4962678-a986-44af-a757-18163a2bc963",
                "p_source_label": "Crowdstrike",
                "timestamp": "2023-05-26 17:59:17.235",
            },
        ),
        RuleTest(
            name="Help command",
            expected_result=False,
            log={
                "aid": "1234abcdefghijklmnop",
                "aip": "1.2.3.4",
                "cid": "abcde098654321xyz",
                "configbuild": "1007.4.0016804.11",
                "configstatehash": "1260279761",
                "entitlements": "15",
                "event": {
                    "CodeSigningFlags": "570506001",
                    "CommandLine": "security help",
                    "ConfigBuild": "1007.4.0016804.11",
                    "ConfigStateHash": "1260279761",
                    "EffectiveTransmissionClass": "2",
                    "Entitlements": "15",
                    "GID": "20",
                    "ImageFileName": "/usr/bin/security",
                    "MD5HashData": "5a29b1ee210395604829dafd744a8039",
                    "MachOSubType": "1",
                    "ParentBaseFileName": "zsh",
                    "ParentProcessId": "488592041865603507",
                    "ProcessEndTime": "",
                    "ProcessGroupId": "488638905187012012",
                    "ProcessStartTime": "1685123957.068",
                    "RGID": "501",
                    "RUID": "501",
                    "RawProcessId": "51673",
                    "SHA1HashData": "0000000000000000000000000000000000000000",
                    "SHA256HashData": "fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de",
                    "SVGID": "20",
                    "SVUID": "501",
                    "SessionProcessId": "488592041748162994",
                    "SigningId": "com.apple.Foundation.plutil",
                    "SourceProcessId": "488592041865603507",
                    "SourceThreadId": "0",
                    "Tags": "316",
                    "TargetProcessId": "488638905187012012",
                    "TeamId": "-",
                    "UID": "501",
                    "aid": "1234abcdefghijklmnop",
                    "aip": "1.2.3.4",
                    "cid": "abcde098654321xyz",
                    "event_platform": "Mac",
                    "event_simpleName": "ProcessRollup2",
                    "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                    "name": "ProcessRollup2MacV9",
                    "timestamp": "1685123957235",
                },
                "event_platform": "Mac",
                "event_simplename": "ProcessRollup2",
                "fdr_event_type": "ProcessRollup2",
                "id": "59c9d5fb-b9b7-43a3-913f-6df72d9e1969",
                "name": "ProcessRollup2MacV9",
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_any_md5_hashes": ["1234abcdefghijklmnop", "5a29b1ee210395604829dafd744a8039", "abcde098654321xyz"],
                "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
                "p_any_sha256_hashes": ["fc708414b7e7c0df8d7711d0362b4dbd79a45fba65bdc646848fa8f50e9035de"],
                "p_any_trace_ids": ["1234abcdefghijklmnop", "abcde098654321xyz"],
                "p_event_time": "2023-05-26 17:59:17.235",
                "p_log_type": "Crowdstrike.FDREvent",
                "p_parse_time": "2023-05-26 18:11:03.404",
                "p_row_id": "d6ff1555b3f6e785b3e2bbb218abf524",
                "p_schema_version": 0,
                "p_source_id": "e4962678-a986-44af-a757-18163a2bc963",
                "p_source_label": "Crowdstrike",
                "timestamp": "2023-05-26 17:59:17.235",
            },
        ),
    ]



import argparse
import json
from typing import Tuple

from pypanther import display, schemas
from pypanther.display import JSON_INDENT_LEVEL
from pypanther.log_types import LogType as ManagedLogType


def run(args: argparse.Namespace) -> Tuple[int, str]:
    log_types = []

    manager = schemas.Manager(args.schemas_path, verbose=False, dry_run=False)
    local_custom_schemas = manager.schemas
    for schema in local_custom_schemas:
        if schema.error:
            return 1, schema.error
        if schema.schema is None:
            raise ValueError("Schema is None")
        if args.substring is None or args.substring.lower() in schema.schema.name.lower():
            log_types.append(schema.schema.name)

    if not args.custom_only:
        for log_type in ManagedLogType:
            if args.substring is None or args.substring.lower() in log_type.lower():
                log_types.append(log_type)

    if args.output == display.OUTPUT_TYPE_TEXT:
        print("\n".join(log_types))
    if args.output == display.OUTPUT_TYPE_JSON:
        print(json.dumps(log_types, indent=JSON_INDENT_LEVEL))

    return 0, ""



"""
Used to define global helpers for Zscaler events
"""


def zia_success(event):
    if (
        event.deep_get("event", "errorcode", default="") == "None"
        and event.deep_get("event", "result", default="") == "SUCCESS"
    ):
        return True
    return False


def zia_alert_context(event):
    event_data = event.get("event", {})
    return {
        "action": event_data.get("action", ""),
        "admin_id": event_data.get("adminid", ""),
        "category": event_data.get("category", ""),
        "client_ip": event_data.get("clientip", ""),
        "preaction": event_data.get("preaction", ""),
        "postaction": event_data.get("postaction", ""),
    }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class OsqueryOutdatedAgent(Rule):
    id = "Osquery.OutdatedAgent-prototype"
    display_name = "Osquery Agent Outdated"
    log_types = [LogType.OSQUERY_DIFFERENTIAL]
    tags = ["Osquery", "Compliance"]
    default_severity = Severity.INFO
    default_description = "Keep track of osquery versions, current is 5.10.2."
    default_runbook = "Update the osquery agent."
    default_reference = "https://www.osquery.io/downloads/official/5.10.2"
    summary_attributes = ["name", "hostIdentifier", "action"]
    LATEST_VERSION = "5.10.2"

    def rule(self, event):
        return (
            event.get("name") == "pack_it-compliance_osquery_info"
            and event.deep_get("columns", "version") != self.LATEST_VERSION
            and (event.get("action") == "added")
        )

    def title(self, event):
        return f"Osquery Version {event.deep_get('columns', 'version')} is Outdated"

    tests = [
        RuleTest(
            name="osquery out of date",
            expected_result=True,
            log={
                "action": "added",
                "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
                "columns": {
                    "build_distro": "10.12",
                    "build_platform": "darwin",
                    "config_hash": "1111",
                    "config_valid": "1",
                    "counter": "14",
                    "global_state": "0",
                    "extensions": "active",
                    "instance_id": "1111",
                    "pid": "223",
                    "resident_size": "54894592",
                    "start_time": "1536634519",
                    "system_time": "12472",
                    "user_time": "31800",
                    "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                    "version": "3.1.2",
                    "watcher": "92",
                },
                "counter": "255",
                "decorations": {"host_uuid": "1111", "environment": "corp"},
                "epoch": "0",
                "hostIdentifier": "test.lan",
                "log_type": "result",
                "name": "pack_it-compliance_osquery_info",
                "unixTime": "1536682461",
            },
        ),
        RuleTest(
            name="osquery up to date",
            expected_result=False,
            log={
                "action": "added",
                "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
                "columns": {
                    "build_distro": "10.12",
                    "build_platform": "darwin",
                    "config_hash": "1111",
                    "config_valid": "1",
                    "counter": "14",
                    "global_state": "2",
                    "extensions": "active",
                    "instance_id": "1111",
                    "pid": "223",
                    "resident_size": "54894592",
                    "start_time": "1536634519",
                    "system_time": "12472",
                    "user_time": "31800",
                    "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                    "version": "5.10.2",
                    "watcher": "92",
                },
                "counter": "255",
                "decorations": {"host_uuid": "1111", "environment": "corp"},
                "epoch": "0",
                "hostIdentifier": "test.lan",
                "log_type": "result",
                "name": "pack_it-compliance_osquery_info",
                "unixTime": "1536682461",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSVPCHealthyLogStatus(Rule):
    id = "AWS.VPC.HealthyLogStatus-prototype"
    display_name = "AWS VPC Healthy Log Status"
    log_types = [LogType.AWS_VPC_FLOW, LogType.OCSF_NETWORK_ACTIVITY]
    tags = ["AWS", "DataModel", "Security Control"]
    default_severity = Severity.INFO
    create_alert = False
    dedup_period_minutes = 1440
    default_description = "Checks for the log status `SKIPDATA`, which indicates that data was lost either to an internal server error or due to capacity constraints.\n"
    default_reference = (
        "https://www.reddit.com/r/aws/comments/zt1xhg/vpc_flow_logs_when_is_logstatus_skipdata_a_concern/?rdt=41505"
    )
    default_runbook = "Determine if the cause of the issue is capacity constraints, and consider adjusting VPC Flow Log configurations accordingly.\n"

    def rule(self, event):
        return event.udm("log_status") == "SKIPDATA"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(name="Healthy Log Status", expected_result=False, log={"status": "OK", "p_log_type": "AWS.VPCFlow"}),
        RuleTest(
            name="Unhealthy Log Status",
            expected_result=True,
            log={"status": "SKIPDATA", "p_log_type": "AWS.VPCFlow"},
        ),
        RuleTest(
            name="Healthy Log Status - OCSF",
            expected_result=False,
            log={"status_code": "OK", "p_log_type": "OCSF.NetworkActivity"},
        ),
        RuleTest(
            name="Unhealthy Log Status - OCSF",
            expected_result=True,
            log={"status_code": "SKIPDATA", "p_log_type": "OCSF.NetworkActivity"},
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSEC2VPCModified(Rule):
    id = "AWS.EC2.VPCModified-prototype"
    display_name = "EC2 VPC Modified"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Security Control", "Defense Evasion:Impair Defenses"]
    reports = {"CIS": ["3.14"], "MITRE ATT&CK": ["TA0005:T1562"]}
    default_severity = Severity.INFO
    dedup_period_minutes = 720
    default_description = "An EC2 VPC was modified."
    default_runbook = "https://docs.runpanther.io/alert-runbooks/built-in-rules/aws-ec2-vpc-modified"
    default_reference = "https://docs.aws.amazon.com/vpc/latest/userguide/configure-your-vpc.html"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    # API calls that are indicative of an EC2 VPC modification
    EC2_VPC_MODIFIED_EVENTS = {
        "CreateVpc",
        "DeleteVpc",
        "ModifyVpcAttribute",
        "AcceptVpcPeeringConnection",
        "CreateVpcPeeringConnection",
        "DeleteVpcPeeringConnection",
        "RejectVpcPeeringConnection",
        "AttachClassicLinkVpc",
        "DetachClassicLinkVpc",
        "DisableVpcClassicLink",
        "EnableVpcClassicLink",
    }

    def rule(self, event):
        return aws_cloudtrail_success(event) and event.get("eventName") in self.EC2_VPC_MODIFIED_EVENTS

    def dedup(self, event):
        return event.get("recipientAccountId")

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="VPC Modified",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "ec2.amazonaws.com",
                "eventName": "CreateVpc",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.ec2.amazonaws.com",
                "requestParameters": {
                    "cidrBlock": "0.0.0.0/26",
                    "instanceTenancy": "default",
                    "amazonProvidedIpv6CidrBlock": False,
                },
                "responseElements": {
                    "requestID": "1",
                    "vpc": {
                        "vpcId": "vpc-1",
                        "state": "pending",
                        "ownerId": "123456789012",
                        "cidrBlock": "0.0.0.0/26",
                        "cidrBlockAssociationSet": {
                            "items": [
                                {
                                    "cidrBlock": "0.0.0.0/26",
                                    "associationId": "vpc-cidr-assoc-1",
                                    "cidrBlockState": {"state": "associated"},
                                },
                            ],
                        },
                        "ipv6CidrBlockAssociationSet": {},
                        "dhcpOptionsId": "dopt-1",
                        "instanceTenancy": "default",
                        "tagSet": {},
                        "isDefault": False,
                    },
                },
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="VPC Not Modified",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "false", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "ec2.amazonaws.com",
                "eventName": "DescribeVpcs",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla",
                "requestParameters": {"vpcSet": {}, "filterSet": {}},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="Error Modifying VPC",
            expected_result=False,
            log={
                "errorCode": "UnknownParameter",
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "1111:tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "ec2.amazonaws.com",
                "eventName": "CreateVpc",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.ec2.amazonaws.com",
                "requestParameters": {
                    "cidrBlock": "0.0.0.0/26",
                    "instanceTenancy": "default",
                    "amazonProvidedIpv6CidrBlock": False,
                },
                "responseElements": {
                    "requestID": "1",
                    "vpc": {
                        "vpcId": "vpc-1",
                        "state": "pending",
                        "ownerId": "123456789012",
                        "cidrBlock": "0.0.0.0/26",
                        "cidrBlockAssociationSet": {
                            "items": [
                                {
                                    "cidrBlock": "0.0.0.0/26",
                                    "associationId": "vpc-cidr-assoc-1",
                                    "cidrBlockState": {"state": "associated"},
                                },
                            ],
                        },
                        "ipv6CidrBlockAssociationSet": {},
                        "dhcpOptionsId": "dopt-1",
                        "instanceTenancy": "default",
                        "tagSet": {},
                        "isDefault": False,
                    },
                },
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class AsanaWorkspacePasswordRequirementsSimple(Rule):
    default_description = "An asana user made your organization's password requirements less strict."
    display_name = "Asana Workspace Password Requirements Simple"
    default_runbook = (
        "Confirm this user acted with valid business intent and determine whether this activity was authorized."
    )
    default_reference = "https://help.asana.com/hc/en-us/articles/14075208738587-Authentication-and-access-management-options-for-paid-plans"
    default_severity = Severity.MEDIUM
    log_types = [LogType.ASANA_AUDIT]
    id = "Asana.Workspace.Password.Requirements.Simple-prototype"

    def rule(self, event):
        new_val = event.deep_get("details", "new_value", default="<NEW_VAL_NOT_FOUND>")
        return all(
            [
                event.get("event_type", "<NO_EVENT_TYPE_FOUND>") == "workspace_password_requirements_changed",
                new_val == "simple",
            ],
        )

    def title(self, event):
        actor_email = event.deep_get("actor", "email", default="<ACTOR_NOT_FOUND>")
        new_value = event.deep_get("details", "new_value", default="<NEW_VAL_NOT_FOUND>")
        old_value = event.deep_get("details", "old_value", default="<OLD_VAL_NOT_FOUND>")
        return f"Asana user [{actor_email}] changed your organization's password requirements from [{old_value}] to [{new_value}]."

    tests = [
        RuleTest(
            name="Simple",
            expected_result=True,
            log={
                "actor": {
                    "actor_type": "user",
                    "email": "homer.simpson@example.io",
                    "gid": "12345",
                    "name": "Homer Simpson",
                },
                "context": {
                    "client_ip_address": "12.12.12.12",
                    "context_type": "web",
                    "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                },
                "created_at": "2022-12-16 19:31:03.667",
                "details": {"new_value": "simple", "old_value": "strong"},
                "event_category": "admin_settings",
                "event_type": "workspace_password_requirements_changed",
                "gid": "12345",
                "resource": {"gid": "12345", "name": "Company Example IO", "resource_type": "workspace"},
            },
        ),
        RuleTest(
            name="web app approvals on",
            expected_result=False,
            log={
                "actor": {
                    "actor_type": "user",
                    "email": "homer.simpson@example.io",
                    "gid": "1234",
                    "name": "Homer Simpson",
                },
                "context": {
                    "client_ip_address": "12.12.12.12",
                    "context_type": "web",
                    "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                },
                "created_at": "2022-12-16 19:29:34.968",
                "details": {"new_value": "all_apps", "old_value": "off"},
                "event_category": "admin_settings",
                "event_type": "workspace_require_app_approvals_of_type_changed",
                "gid": "1234",
                "resource": {"gid": "1234", "name": "Panther Labs", "resource_type": "workspace"},
            },
        ),
    ]



import re
from fnmatch import fnmatch

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.base import EMAIL_REGEX
from pypanther.helpers.msft import m365_alert_context


@panther_managed
class Microsoft365ExternalDocumentSharing(Rule):
    default_description = "Document shared externally"
    display_name = "Microsoft365 External Document Sharing"
    reports = {"MITRE ATT&CK": ["TA0009:T1039"]}
    default_runbook = "Check the document metadata to ensure it is not a sensitive document."
    default_reference = "https://support.microsoft.com/en-us/topic/manage-sharing-with-external-users-in-microsoft-365-small-business-2951a85f-c970-4375-aa4f-6b0d7035fe35#:~:text=Top%20of%20Page-,Turn%20external%20sharing%20on%20or%20off,-The%20ability%20to"
    default_severity = Severity.LOW
    log_types = [LogType.MICROSOFT365_AUDIT_SHAREPOINT]
    id = "Microsoft365.External.Document.Sharing-prototype"
    ALLOWED_DOMAINS = ["mycompany.com", "alloweddomain.com"]  # should be in lowercase
    ALLOWED_USERS = ["exception@outsider.com"]  # should be in lowercase
    ALLOWED_PATHS = ["*/External/*", "External/*"]

    def allowed_path(self, relative_url):
        for path in self.ALLOWED_PATHS:
            if fnmatch(relative_url, path):
                return True
        return False

    def rule(self, event):
        if event.get("Operation", "") == "AnonymousLinkCreated":
            return not self.allowed_path(event.get("SourceRelativeUrl"))
        if event.get("Operation", "") == "AddedToSecureLink":
            if self.allowed_path(event.get("SourceRelativeUrl")):
                return False
            target = event.get("TargetUserOrGroupName", "")
            if target.lower() in self.ALLOWED_USERS:
                return False
            if re.fullmatch(EMAIL_REGEX, target):
                if target.split("@")[1].lower() not in self.ALLOWED_DOMAINS:
                    return True
        return False

    def title(self, event):
        return f"Microsoft365: [{event.get('SourceRelativeUrl')}] has been shared with external users by [{event.get('UserId', '<user-not-found>')}]"

    def alert_context(self, event):
        return m365_alert_context(event)

    tests = [
        RuleTest(
            name="Allowed Domain",
            expected_result=False,
            log={
                "AppAccessContext": {"AADSessionId": "aa-bb-cc", "CorrelationId": "dd-ee-ff"},
                "ClientIP": "1.2.3.4",
                "CreationTime": "2022-12-12 19:31:41",
                "EventData": "<Type>Edit</Type><MembersCanShareApplied>False</MembersCanShareApplied>",
                "EventSource": "SharePoint",
                "Id": "111-aa-234",
                "ItemType": "File",
                "ObjectId": "https://yourorg.sharepoint.com/personal/user_yourorg/Documents/importantsecrets.docx",
                "Operation": "AddedToSecureLink",
                "OrganizationId": "11-22-abc",
                "RecordType": 14,
                "Site": "aa-bb-dd-ee-ff",
                "SiteUrl": "https://yourorg.sharepoint.com/personal/user_yourorg",
                "SourceFileExtension": "docx",
                "SourceFileName": "importantsecrets.docx",
                "SourceRelativeUrl": "Documents/importantsecrets.docx",
                "TargetUserOrGroupName": "OUTSIDER@ALLOWEDDOMAIN.COM",
                "TargetUserOrGroupType": "Guest",
                "UserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                "UserId": "my.user@yourorg.onmicrosoft.com",
                "UserKey": "i:0h.f|membership|12345@live.com",
                "UserType": 0,
                "Workload": "OneDrive",
            },
        ),
        RuleTest(
            name="Allowed Folder",
            expected_result=False,
            log={
                "AppAccessContext": {"AADSessionId": "aa-bb-cc", "CorrelationId": "dd-ee-ff"},
                "ClientIP": "1.2.3.4",
                "CreationTime": "2022-12-12 19:31:41",
                "EventData": "<Type>Edit</Type><MembersCanShareApplied>False</MembersCanShareApplied>",
                "EventSource": "SharePoint",
                "Id": "111-aa-234",
                "ItemType": "File",
                "ObjectId": "https://yourorg.sharepoint.com/personal/user_yourorg/Documents/External/public.docx",
                "Operation": "AddedToSecureLink",
                "OrganizationId": "11-22-abc",
                "RecordType": 14,
                "Site": "aa-bb-dd-ee-ff",
                "SiteUrl": "https://yourorg.sharepoint.com/personal/user_yourorg",
                "SourceFileExtension": "docx",
                "SourceFileName": "public.docx",
                "SourceRelativeUrl": "Documents/External/public.docx",
                "TargetUserOrGroupName": "MARKETING@SAAS.COM",
                "TargetUserOrGroupType": "Guest",
                "UserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                "UserId": "my.user@yourorg.onmicrosoft.com",
                "UserKey": "i:0h.f|membership|12345@live.com",
                "UserType": 0,
                "Workload": "OneDrive",
            },
        ),
        RuleTest(
            name="Allowed User",
            expected_result=False,
            log={
                "AppAccessContext": {"AADSessionId": "aa-bb-cc", "CorrelationId": "dd-ee-ff"},
                "ClientIP": "1.2.3.4",
                "CreationTime": "2022-12-12 19:31:41",
                "EventData": "<Type>Edit</Type><MembersCanShareApplied>False</MembersCanShareApplied>",
                "EventSource": "SharePoint",
                "Id": "111-aa-234",
                "ItemType": "File",
                "ObjectId": "https://yourorg.sharepoint.com/personal/user_yourorg/Documents/importantsecrets.docx",
                "Operation": "AddedToSecureLink",
                "OrganizationId": "11-22-abc",
                "RecordType": 14,
                "Site": "aa-bb-dd-ee-ff",
                "SiteUrl": "https://yourorg.sharepoint.com/personal/user_yourorg",
                "SourceFileExtension": "docx",
                "SourceFileName": "importantsecrets.docx",
                "SourceRelativeUrl": "Documents/importantsecrets.docx",
                "TargetUserOrGroupName": "EXCEPTION@OUTSIDER.COM",
                "TargetUserOrGroupType": "Guest",
                "UserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                "UserId": "my.user@yourorg.onmicrosoft.com",
                "UserKey": "i:0h.f|membership|12345@live.com",
                "UserType": 0,
                "Workload": "OneDrive",
            },
        ),
        RuleTest(
            name="External user",
            expected_result=True,
            log={
                "AppAccessContext": {"AADSessionId": "aa-bb-cc", "CorrelationId": "dd-ee-ff"},
                "ClientIP": "1.2.3.4",
                "CreationTime": "2022-12-12 19:31:41",
                "EventData": "<Type>Edit</Type><MembersCanShareApplied>False</MembersCanShareApplied>",
                "EventSource": "SharePoint",
                "Id": "111-aa-234",
                "ItemType": "File",
                "ObjectId": "https://yourorg.sharepoint.com/personal/user_yourorg/Documents/importantsecrets.docx",
                "Operation": "AddedToSecureLink",
                "OrganizationId": "11-22-abc",
                "RecordType": 14,
                "Site": "aa-bb-dd-ee-ff",
                "SiteUrl": "https://yourorg.sharepoint.com/personal/user_yourorg",
                "SourceFileExtension": "docx",
                "SourceFileName": "importantsecrets.docx",
                "SourceRelativeUrl": "Documents/importantsecrets.docx",
                "TargetUserOrGroupName": "OUTSIDER@EXTERNAL.IO",
                "TargetUserOrGroupType": "Guest",
                "UserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                "UserId": "my.user@yourorg.onmicrosoft.com",
                "UserKey": "i:0h.f|membership|12345@live.com",
                "UserType": 0,
                "Workload": "OneDrive",
            },
        ),
        RuleTest(
            name="Internal Share",
            expected_result=False,
            log={
                "AppAccessContext": {"AADSessionId": "aa-bb-cc", "CorrelationId": "dd-ee-ff"},
                "ClientIP": "1.2.3.4",
                "CreationTime": "2022-12-12 19:31:41",
                "EventData": "<Type>Edit</Type><MembersCanShareApplied>False</MembersCanShareApplied>",
                "EventSource": "SharePoint",
                "Id": "111-aa-234",
                "ItemType": "File",
                "ObjectId": "https://yourorg.sharepoint.com/personal/user_yourorg/Documents/importantsecrets.docx",
                "Operation": "AddedToSecureLink",
                "OrganizationId": "11-22-abc",
                "RecordType": 14,
                "Site": "aa-bb-dd-ee-ff",
                "SiteUrl": "https://yourorg.sharepoint.com/personal/user_yourorg",
                "SourceFileExtension": "docx",
                "SourceFileName": "importantsecrets.docx",
                "SourceRelativeUrl": "Documents/importantsecrets.docx",
                "TargetUserOrGroupName": "ANOTHERUSER@MYCOMPANY.COM",
                "TargetUserOrGroupType": "NotGuest",
                "UserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                "UserId": "my.user@yourorg.onmicrosoft.com",
                "UserKey": "i:0h.f|membership|12345@live.com",
                "UserType": 0,
                "Workload": "OneDrive",
            },
        ),
    ]



from ipaddress import ip_network

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSVPCInboundPortWhitelist(Rule):
    id = "AWS.VPC.InboundPortWhitelist-prototype"
    display_name = "VPC Flow Logs Inbound Port Allowlist"
    enabled = False
    log_types = [LogType.AWS_VPC_FLOW, LogType.OCSF_NETWORK_ACTIVITY]
    tags = ["AWS", "DataModel", "Configuration Required", "Security Control", "Command and Control:Non-Standard Port"]
    reports = {"MITRE ATT&CK": ["TA0011:T1571"]}
    default_reference = "https://docs.aws.amazon.com/vpc/latest/userguide/flow-logs.html"
    default_severity = Severity.HIGH
    default_description = "VPC Flow Logs observed inbound traffic violating the port allowlist.\n"
    default_runbook = "Block the unapproved traffic, or update the approved ports list.\n"
    summary_attributes = ["srcaddr", "dstaddr", "dstport"]
    APPROVED_PORTS = {80, 443}

    def rule(self, event):
        # Can't perform this check without a destination port
        if not event.udm("destination_port"):
            return False
        # Only monitor for non allowlisted ports
        if event.udm("destination_port") in self.APPROVED_PORTS:
            return False
        # Only monitor for traffic coming from non-private IP space
        #
        # Defaults to True (no alert) if 'srcaddr' key is not present
        source_ip = event.udm("source_ip") or "0.0.0.0/32"
        if not ip_network(source_ip).is_global:
            return False
        # Alert if the traffic is destined for internal IP addresses
        #
        # Defaults to False (no alert) if 'dstaddr' key is not present
        destination_ip = event.udm("destination_ip") or "1.0.0.0/32"
        return not ip_network(destination_ip).is_global

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Public to Private IP on Restricted Port",
            expected_result=True,
            log={"dstPort": 22, "dstAddr": "10.0.0.1", "srcAddr": "1.1.1.1", "p_log_type": "AWS.VPCFlow"},
        ),
        RuleTest(
            name="Public to Private IP on Allowed Port",
            expected_result=False,
            log={"dstPort": 443, "dstAddr": "10.0.0.1", "srcAddr": "1.1.1.1", "p_log_type": "AWS.VPCFlow"},
        ),
        RuleTest(
            name="Private to Private IP on Restricted Port",
            expected_result=False,
            log={"dstPort": 22, "dstAddr": "10.0.0.1", "srcAddr": "10.10.10.1", "p_log_type": "AWS.VPCFlow"},
        ),
        RuleTest(
            name="Public to Private IP on Restricted Port - OCSF",
            expected_result=True,
            log={
                "dst_endpoint": {"ip": "10.0.0.1", "port": 22},
                "src_endpoint": {"ip": "1.1.1.1"},
                "p_log_type": "OCSF.NetworkActivity",
            },
        ),
        RuleTest(
            name="Public to Private IP on Allowed Port - OCSF",
            expected_result=False,
            log={
                "dst_endpoint": {"ip": "10.0.0.1", "port": 443},
                "src_endpoint": {"ip": "1.1.1.1"},
                "p_log_type": "OCSF.NetworkActivity",
            },
        ),
    ]



"""
Used to define global helpers and variables for GitHub events.
"""


def github_alert_context(event):
    return {
        "action": event.get("action", ""),
        "actor": event.get("actor", ""),
        "actor_location": event.deep_get("actor_location", "country_code"),
        "org": event.get("org", ""),
        "repo": event.get("repo", ""),
        "user": event.get("user", ""),
    }



from pypanther.rules.standard.admin_assigned import StandardAdminRoleAssigned as StandardAdminRoleAssigned
from pypanther.rules.standard.brute_force_by_ip import StandardBruteForceByIP as StandardBruteForceByIP
from pypanther.rules.standard.brute_force_by_user import StandardBruteForceByUser as StandardBruteForceByUser
from pypanther.rules.standard.impossible_travel_login import (
    StandardImpossibleTravelLogin as StandardImpossibleTravelLogin,
)
from pypanther.rules.standard.malicious_sso_dns_lookup import (
    StandardMaliciousSSODNSLookup as StandardMaliciousSSODNSLookup,
)
from pypanther.rules.standard.mfa_disabled import StandardMFADisabled as StandardMFADisabled
from pypanther.rules.standard.sign_in_from_rogue_state import (
    StandardSignInFromRogueState as StandardSignInFromRogueState,
)
from pypanther.rules.standard.standard_dns_base64 import StandardDNSBase64 as StandardDNSBase64



"""
Global helpers for Snowflake streaming detections
"""

""" Global helpers for Snowflake streaming detections. """


def query_history_alert_context(event):
    return {
        "user": event.get("user_name", "<UNKNOWN USER>"),
        "role": event.get("role_name", "<UNKNOWN ROLE>"),
        "source": event.get("p_source_label", "<UNKNOWN SOURCE>"),
        # Not all queries are run in a warehouse; e.g.: getting worksheet files
        "warehouse": event.get("WAREHOUSE_NAME", "<NO WAREHOUSE>"),
    }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp import gcp_alert_context


@panther_managed
class GCPDNSZoneModifiedorDeleted(Rule):
    default_description = "Detection for GCP DNS zones that are deleted, patched, or updated."
    display_name = "GCP DNS Zone Modified or Deleted"
    default_runbook = "Verify that this modification or deletion was expected. These operations are high-impact events and can result in downtimes or total outages."
    default_reference = "https://cloud.google.com/dns/docs/zones"
    default_severity = Severity.LOW
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.DNS.Zone.Modified.or.Deleted-prototype"

    def rule(self, event):
        methods = ("dns.changes.create", "dns.managedZones.delete", "dns.managedZones.patch", "dns.managedZones.update")
        return event.deep_get("protoPayload", "methodName", default="") in methods

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        resource = event.deep_get("protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>")
        return f"[GCP]: [{actor}] modified managed DNS zone [{resource}]"

    def alert_context(self, event):
        return gcp_alert_context(event)

    tests = [
        RuleTest(
            name="dns.managedZones.delete-should-alert",
            expected_result=True,
            log={
                "insertid": "-xxxxxxxxxxxx",
                "logName": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user@domain.com"},
                    "authorizationInfo": [
                        {"granted": True, "permission": "dns.managedZones.delete", "resourceAttributes": {}},
                    ],
                    "methodName": "dns.managedZones.delete",
                    "request": {
                        "@type": "type.googleapis.com/cloud.dns.api.ManagedZonesDeleteRequest",
                        "managedZone": "test-zone",
                        "project": "test-project-123456",
                    },
                    "requestMetadata": {
                        "callerIP": "12.12.12.12",
                        "destinationAttributes": {},
                        "requestAttributes": {"auth": {}, "time": "2023-05-23T19:08:13.820007Z"},
                    },
                    "resourceName": "managedZones/test-zone",
                    "response": {"@type": "type.googleapis.com/cloud.dns.api.ManagedZonesDeleteResponse"},
                    "serviceName": "dns.googleapis.com",
                    "status": {},
                },
                "receivetimestamp": "2023-05-23 19:08:14.305",
                "resource": {
                    "labels": {"location": "global", "project_id": "test-project-123456", "zone_name": "test-zone"},
                    "type": "dns_managed_zone",
                },
                "severity": "NOTICE",
                "timestamp": "2023-05-23 19:08:11.697",
            },
        ),
        RuleTest(
            name="dns.managedZones.patch-should-alert",
            expected_result=True,
            log={
                "insertid": "-xxxxxxxxxxxx",
                "logname": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user@domain.com"},
                    "authorizationInfo": [
                        {"granted": True, "permission": "dns.managedZones.update", "resourceAttributes": {}},
                    ],
                    "methodName": "dns.managedZones.patch",
                    "request": {
                        "@type": "type.googleapis.com/cloud.dns.api.ManagedZonesPatchRequest",
                        "managedZone": "test-zone",
                        "managedZoneResource": {
                            "description": "testing",
                            "privateVisibilityConfig": {
                                "networks": [
                                    {
                                        "networkUrl": "https://www.googleapis.com/compute/v1/projects/test-project-123456/global/networks/default",
                                    },
                                ],
                            },
                        },
                        "project": "test-project-123456",
                    },
                    "requestMetadata": {
                        "callerIP": "12.12.12.12",
                        "destinationAttributes": {},
                        "requestAttributes": {"auth": {}, "time": "2023-05-23T19:07:25.568071Z"},
                    },
                    "resourceName": "managedZones/test-zone",
                    "response": {
                        "@type": "type.googleapis.com/cloud.dns.api.ManagedZonesPatchResponse",
                        "managedZone": {
                            "cloudLoggingConfig": {},
                            "creationTime": "2023-05-23T18:59:57.919Z",
                            "description": "testing",
                            "dnsName": "test.detectiontesting.com.",
                            "fingerprint": "3f961d0b0a9e6a8c000001884a024eed",
                            "id": "4581881604156058252",
                            "name": "test-zone",
                            "nameServers": ["ns-gcp-private.googledomains.com."],
                            "privateVisibilityConfig": {
                                "networks": [
                                    {
                                        "networkUrl": "https://www.googleapis.com/compute/v1/projects/test-project-123456/global/networks/default",
                                    },
                                ],
                            },
                            "rrsetCount": 2,
                            "visibility": "PRIVATE",
                        },
                        "operation": {
                            "id": "a7513a2c-e637-4b86-b223-1c4f8b0797be",
                            "startTime": "2023-05-23T19:07:25.511Z",
                            "status": "DONE",
                            "type": "UPDATE",
                            "user": "user@domain.com",
                            "zoneContext": {
                                "newValue": {
                                    "cloudLoggingConfig": {},
                                    "creationTime": "2023-05-23T18:59:57.919Z",
                                    "description": "testing",
                                    "dnsName": "test.detectiontesting.com.",
                                    "fingerprint": "3f961d0b0a9e6a8c000001884a024eed",
                                    "id": "4581881604156058252",
                                    "name": "test-zone",
                                    "nameServers": ["ns-gcp-private.googledomains.com."],
                                    "privateVisibilityConfig": {
                                        "networks": [
                                            {
                                                "networkUrl": "https://www.googleapis.com/compute/v1/projects/test-project-123456/global/networks/default",
                                            },
                                        ],
                                    },
                                    "rrsetCount": 2,
                                    "visibility": "PRIVATE",
                                },
                                "oldValue": {
                                    "cloudLoggingConfig": {},
                                    "creationTime": "2023-05-23T18:59:57.919Z",
                                    "description": "testing",
                                    "dnsName": "test.detectiontesting.com.",
                                    "fingerprint": "3f961d0b0a9e6a8c0000018849fb7b5f",
                                    "id": "4581881604156058252",
                                    "name": "test-zone",
                                    "nameServers": ["ns-gcp-private.googledomains.com."],
                                    "rrsetCount": 2,
                                    "visibility": "PRIVATE",
                                },
                            },
                        },
                    },
                    "serviceName": "dns.googleapis.com",
                    "status": {},
                },
                "receivetimestamp": "2023-05-23 19:07:26.276",
                "resource": {
                    "labels": {"location": "global", "project_id": "test-project-123456", "zone_name": "test-zone"},
                    "type": "dns_managed_zone",
                },
                "severity": "NOTICE",
                "timestamp": "2023-05-23 19:07:25.282",
            },
        ),
        RuleTest(
            name="dns.managedZones.update-should-alert",
            expected_result=True,
            log={
                "insertid": "xxxxxxxxxxxx",
                "logname": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user@domain.com"},
                    "authorizationInfo": [
                        {"granted": True, "permission": "dns.changes.create", "resourceAttributes": {}},
                    ],
                    "methodName": "dns.changes.create",
                    "request": {
                        "@type": "type.googleapis.com/cloud.dns.api.ChangesCreateRequest",
                        "change": {
                            "additions": [
                                {
                                    "name": "test.detectiontesting.com.",
                                    "rrdata": [
                                        "ns-gcp-private.googledomains.com. cloud-dns-hostmaster.google.com. 1 21600 3600 259200 300",
                                    ],
                                    "ttl": 3600,
                                    "type": "SOA",
                                },
                            ],
                            "deletions": [
                                {
                                    "name": "test.detectiontesting.com.",
                                    "rrdata": [
                                        "ns-gcp-private.googledomains.com. cloud-dns-hostmaster.google.com. 1 21600 3600 259200 300",
                                    ],
                                    "ttl": 21600,
                                    "type": "SOA",
                                },
                            ],
                        },
                        "managedZone": "test-zone",
                        "project": "test-project-123456",
                    },
                    "requestMetadata": {
                        "callerIP": "12.12.12.12",
                        "destinationAttributes": {},
                        "requestAttributes": {"auth": {}, "time": "2023-05-23T19:07:39.239275Z"},
                    },
                    "resourceName": "managedZones/test-zone",
                    "response": {
                        "@type": "type.googleapis.com/cloud.dns.api.ChangesCreateResponse",
                        "change": {
                            "additions": [
                                {
                                    "name": "test.detectiontesting.com.",
                                    "rrdata": [
                                        "ns-gcp-private.googledomains.com. cloud-dns-hostmaster.google.com. 1 21600 3600 259200 300",
                                    ],
                                    "ttl": 3600,
                                    "type": "SOA",
                                },
                            ],
                            "deletions": [
                                {
                                    "name": "test.detectiontesting.com.",
                                    "rrdata": [
                                        "ns-gcp-private.googledomains.com. cloud-dns-hostmaster.google.com. 1 21600 3600 259200 300",
                                    ],
                                    "ttl": 21600,
                                    "type": "SOA",
                                },
                            ],
                            "id": "1",
                            "startTime": "2023-05-23T19:07:39.155Z",
                            "status": "PENDING",
                        },
                    },
                    "serviceName": "dns.googleapis.com",
                    "status": {},
                },
                "receivetimestamp": "2023-05-23 19:07:40.053",
                "resource": {
                    "labels": {"location": "global", "project_id": "test-project-123456", "zone_name": "test-zone"},
                    "type": "dns_managed_zone",
                },
                "severity": "NOTICE",
                "timestamp": "2023-05-23 19:07:39.132",
            },
        ),
        RuleTest(
            name="dns.managedZones.get-should-not-alert",
            expected_result=False,
            log={
                "insertid": "-nkgd1se1zsiw",
                "logName": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "staging@pantherstaging.io"},
                    "authorizationInfo": [
                        {"granted": True, "permission": "dns.managedZones.get", "resourceAttributes": {}},
                    ],
                    "methodName": "dns.managedZones.get",
                    "request": {
                        "@type": "type.googleapis.com/cloud.dns.api.ManagedZonesGetRequest",
                        "managedZone": "test-zone",
                        "project": "test-project-123456",
                    },
                    "requestMetadata": {
                        "callerIP": "12.12.12.12",
                        "destinationAttributes": {},
                        "requestAttributes": {"auth": {}, "time": "2023-05-23T19:08:13.820007Z"},
                    },
                    "resourceName": "managedZones/test-zone",
                    "response": {"@type": "type.googleapis.com/cloud.dns.api.ManagedZonesGetResponse"},
                    "serviceName": "dns.googleapis.com",
                    "status": {},
                },
                "receivetimestamp": "2023-05-23 19:08:14.305",
                "resource": {
                    "labels": {"location": "global", "project_id": "test-project-123456", "zone_name": "test-zone"},
                    "type": "dns_managed_zone",
                },
                "severity": "NOTICE",
                "timestamp": "2023-05-23 19:08:11.697",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.zscaler import zia_alert_context, zia_success


@panther_managed
class ZIAInsecurePasswordSettings(Rule):
    id = "ZIA.Insecure.Password.Settings-prototype"
    default_description = "This rule detects when password settings are insecure."
    display_name = "ZIA Insecure Password Settings"
    default_runbook = "Set the secure password configurations."
    default_reference = "https://help.zscaler.com/zia/customizing-your-admin-account-settings"
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0005:T1556.009"]}
    log_types = [LogType.ZSCALER_ZIA_ADMIN_AUDIT_LOG]

    def rule(self, event):
        if not zia_success(event):
            return False
        auth_frequency = event.deep_get("event", "postaction", "authFrequency", default="<AUTH_FREQUENCY_NOT_FOUND>")
        password_expiry = event.deep_get("event", "postaction", "passwordExpiry", default="<PASSWORD_EXPIRY_NOT_FOUND>")
        password_strength = event.deep_get(
            "event",
            "postaction",
            "passwordStrength",
            default="<PASSWORD_STRENGTH_NOT_FOUND>",
        )
        if auth_frequency == "PERMANENT_COOKIE" or password_expiry == "NEVER" or password_strength == "NONE":  # nosec bandit B105
            # nosec bandit B105
            return True
        return False

    def dedup(self, event):
        return event.deep_get("event", "adminid", default="<ADMIN_ID_NOT_FOUND>")

    def title(self, event):
        return f"[Zscaler.ZIA]: Password settings are insecure for admin with id [{event.deep_get('event', 'adminid', default='<ADMIN_ID_NOT_FOUND>')}]"

    def alert_context(self, event):
        return zia_alert_context(event)

    tests = [
        RuleTest(
            name="Permanent cookie",
            expected_result=True,
            log={
                "event": {
                    "action": "UPDATE",
                    "adminid": "admin@test.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "AUTHENTICATION_SETTINGS",
                    "clientip": "1.2.3.4",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {
                        "authFrequency": "PERMANENT_COOKIE",
                        "autoProvision": False,
                        "directorySyncMigrateToScimEnabled": False,
                        "kerberosEnabled": False,
                        "mobileAdminSamlIdpEnabled": False,
                        "oneTimeAuth": "OTP_DISABLED",
                        "orgAuthType": "SAFECHANNEL_DIR",
                        "passwordExpiry": "NEVER",
                        "passwordStrength": "NONE",
                        "samlEnabled": False,
                    },
                    "preaction": {
                        "authFrequency": "DAILY_COOKIE",
                        "autoProvision": False,
                        "directorySyncMigrateToScimEnabled": False,
                        "kerberosEnabled": False,
                        "mobileAdminSamlIdpEnabled": False,
                        "oneTimeAuth": "OTP_DISABLED",
                        "orgAuthType": "SAFECHANNEL_DIR",
                        "passwordExpiry": "NEVER",
                        "passwordStrength": "NONE",
                        "samlEnabled": False,
                    },
                    "recordid": "356",
                    "resource": "None",
                    "result": "SUCCESS",
                    "subcategory": "AUTH_SETTINGS_PROFILE",
                    "time": "2024-11-04 16:29:24.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
        RuleTest(
            name="Password expiry - never",
            expected_result=True,
            log={
                "event": {
                    "action": "UPDATE",
                    "adminid": "admin@test.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "AUTHENTICATION_SETTINGS",
                    "clientip": "1.2.3.4",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {
                        "authFrequency": "DAILY_COOKIE",
                        "autoProvision": False,
                        "directorySyncMigrateToScimEnabled": False,
                        "kerberosEnabled": False,
                        "mobileAdminSamlIdpEnabled": False,
                        "oneTimeAuth": "OTP_LINK",
                        "orgAuthType": "SAFECHANNEL_DIR",
                        "passwordExpiry": "NEVER",
                        "passwordStrength": "NONE",
                        "samlEnabled": False,
                    },
                    "preaction": {
                        "authFrequency": "DAILY_COOKIE",
                        "autoProvision": False,
                        "directorySyncMigrateToScimEnabled": False,
                        "kerberosEnabled": False,
                        "mobileAdminSamlIdpEnabled": False,
                        "oneTimeAuth": "OTP_DISABLED",
                        "orgAuthType": "SAFECHANNEL_DIR",
                        "passwordExpiry": "NEVER",
                        "passwordStrength": "NONE",
                        "samlEnabled": False,
                    },
                    "recordid": "357",
                    "resource": "None",
                    "result": "SUCCESS",
                    "subcategory": "AUTH_SETTINGS_PROFILE",
                    "time": "2024-11-04 16:29:40.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
        RuleTest(
            name="Password strength - none",
            expected_result=True,
            log={
                "event": {
                    "action": "UPDATE",
                    "adminid": "admin@test.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "AUTHENTICATION_SETTINGS",
                    "clientip": "1.2.3.4",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {
                        "authFrequency": "DAILY_COOKIE",
                        "autoProvision": False,
                        "directorySyncMigrateToScimEnabled": False,
                        "kerberosEnabled": False,
                        "mobileAdminSamlIdpEnabled": False,
                        "oneTimeAuth": "OTP_DISABLED",
                        "orgAuthType": "SAFECHANNEL_DIR",
                        "passwordExpiry": "SIX_MONTHS",
                        "passwordStrength": "NONE",
                        "samlEnabled": False,
                    },
                    "preaction": {
                        "authFrequency": "DAILY_COOKIE",
                        "autoProvision": False,
                        "directorySyncMigrateToScimEnabled": False,
                        "kerberosEnabled": False,
                        "mobileAdminSamlIdpEnabled": False,
                        "oneTimeAuth": "OTP_DISABLED",
                        "orgAuthType": "SAFECHANNEL_DIR",
                        "passwordExpiry": "NEVER",
                        "passwordStrength": "NONE",
                        "samlEnabled": False,
                    },
                    "recordid": "361",
                    "resource": "None",
                    "result": "SUCCESS",
                    "subcategory": "AUTH_SETTINGS_PROFILE",
                    "time": "2024-11-04 16:30:36.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
        RuleTest(
            name="Other event",
            expected_result=False,
            log={
                "event": {
                    "action": "SIGN_IN",
                    "adminid": "admin@test.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "LOGIN",
                    "clientip": "1.2.3.4",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {},
                    "preaction": {},
                    "recordid": "354",
                    "resource": "None",
                    "result": "SUCCESS",
                    "subcategory": "LOGIN",
                    "time": "2024-11-04 16:27:37.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
    ]



import json

from policyuniverse.policy import Policy

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context
from pypanther.helpers.base import deep_get


@panther_managed
class AWSCloudTrailResourceMadePublic(Rule):
    id = "AWS.CloudTrail.ResourceMadePublic-prototype"
    display_name = "AWS Resource Made Public"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Exfiltration:Transfer Data to Cloud Account"]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0010:T1537"]}
    default_description = "Some AWS resource was made publicly accessible over the internet. Checks ECR, Elasticsearch, KMS, S3, S3 Glacier, SNS, SQS, and Secrets Manager.\n"
    default_runbook = "Adjust the policy so that the resource is no longer publicly accessible"
    default_reference = "https://aws.amazon.com/blogs/security/identifying-publicly-accessible-resources-with-amazon-vpc-network-access-analyzer/"
    summary_attributes = ["userAgent", "sourceIpAddress", "vpcEndpointId", "recipientAccountId", "p_any_aws_arns"]
    # Check if a policy (string or JSON) allows resource accessibility via the Internet
    # pylint: disable=too-complex

    def policy_is_internet_accessible(self, policy):
        """
        Check if a policy (string or JSON) allows resource accessibility via the Internet.

        Args:
        ----
        policy: A policy object that can be either a string or a JSON object

        Returns:
        -------
        bool: True if the policy allows internet access, False otherwise

        """
        # Handle empty policies (None, empty strings, empty dicts, etc.)
        if not policy:
            return False
        # Handle string policies by converting to JSON
        if isinstance(policy, str):
            try:
                policy = json.loads(policy)
            except json.JSONDecodeError:
                return False
        # Check if the policy has a wildcard principal but also has organization ID restrictions
        # which should not be considered internet accessible
        policy_obj = Policy(policy)
        # If policyuniverse thinks it's not internet accessible, trust that
        if not policy_obj.is_internet_accessible():
            return False
        # For policies with multiple statements, we need to check each statement individually
        # If ANY statement is truly internet accessible, the policy is internet accessible
        has_internet_accessible_statement = False
        for statement in policy_obj.statements:
            if statement.effect != "Allow" or "*" not in statement.principals:
                continue
            # Check if there are organization ID conditions which restrict access
            has_org_condition = False
            for condition in statement.condition_entries:
                if condition.category == "organization":
                    has_org_condition = True
                    break
            # If this statement has a wildcard principal but no organization ID restrictions,
            # it's truly internet accessible
            if not has_org_condition:
                has_internet_accessible_statement = True
                break
        return has_internet_accessible_statement

    def rule(self, event):
        if not aws_cloudtrail_success(event):
            return False
        parameters = event.get("requestParameters", {})
        # Ignore events that are missing request params
        if not parameters:
            return False
        event_name = event.get("eventName", "")
        # Special case for SNS topic attributes that need additional attribute name check
        if event_name == "SetTopicAttributes" and parameters.get("attributeName", "") == "Policy":
            policy_value = parameters.get("attributeValue", {})
            return self.policy_is_internet_accessible(policy_value)
        # Map of event names to policy locations in parameters
        # S3
        # ECR
        # Elasticsearch
        # KMS
        # S3 Glacier
        # SNS & SQS
        # SecretsManager
        policy_location_map = {
            "PutBucketPolicy": lambda p: p.get("bucketPolicy", {}),
            "SetRepositoryPolicy": lambda p: p.get("policyText", {}),
            "CreateElasticsearchDomain": lambda p: p.get("accessPolicies", {}),
            "UpdateElasticsearchDomainConfig": lambda p: p.get("accessPolicies", {}),
            "CreateKey": lambda p: p.get("policy", {}),
            "PutKeyPolicy": lambda p: p.get("policy", {}),
            "SetVaultAccessPolicy": lambda p: deep_get(p, "policy", "policy", default={}),
            "SetQueueAttributes": lambda p: deep_get(p, "attributes", "Policy", default={}),
            "CreateTopic": lambda p: deep_get(p, "attributes", "Policy", default={}),
            "PutResourcePolicy": lambda p: p.get("resourcePolicy", {}),
        }
        # Get the policy extraction function for this event name
        policy_extractor = policy_location_map.get(event_name)
        if not policy_extractor:
            return False
        # Extract the policy using the appropriate function
        policy = policy_extractor(parameters)
        return self.policy_is_internet_accessible(policy)

    def title(self, event):
        # TODO(): Update this rule to use data models
        user = event.deep_get("userIdentity", "userName") or event.deep_get(
            "userIdentity",
            "sessionContext",
            "sessionIssuer",
            "userName",
            default="<MISSING_USER>",
        )
        if event.get("Resources"):
            return f"Resource {event.get('Resources')[0].get('arn', 'MISSING')} made public by {user}"
        return f"{event.get('eventSource', 'MISSING SOURCE')} resource made public by {user}"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="ECR Made Public",
            expected_result=True,
            log={
                "awsRegion": "eu-west-1",
                "eventID": "685e066d-a3aa-4323-a6a1-2f187a2fc986",
                "eventName": "SetRepositoryPolicy",
                "eventSource": "ecr.amazonaws.com",
                "eventTime": "2020-11-20 06:19:05.000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.05",
                "recipientAccountId": "112233445566",
                "requestID": "95fd6392-627c-467b-b940-895183d3298d",
                "requestParameters": {
                    "force": False,
                    "policyText": '{"Version":"2012-10-17","Statement":[{"Action":["ecr:BatchCheckLayerAvailability","ecr:BatchGetImage","ecr:GetAuthorizationToken","ecr:GetDownloadUrlForLayer"],"Effect":"Allow","Principal":"*","Sid":"PublicRead"}]}',
                    "repositoryName": "community",
                },
                "resources": [
                    {"accountId": "112233445566", "arn": "arn:aws:ecr:eu-west-1:112233445566:repository/community"},
                ],
                "responseElements": {
                    "policyText": '{\n  "Version" : "2012-10-17",\n  "Statement" : [ {\n    "Sid" : "PublicRead",\n    "Effect" : "Allow",\n    "Principal" : "*",\n    "Action" : [ "ecr:BatchCheckLayerAvailability", "ecr:BatchGetImage", "ecr:GetAuthorizationToken", "ecr:GetDownloadUrlForLayer" ]\n  } ]\n}',
                    "registryId": "112233445566",
                    "repositoryName": "community",
                },
                "sourceIPAddress": "cloudformation.amazonaws.com",
                "userAgent": "cloudformation.amazonaws.com",
                "userIdentity": {
                    "accessKeyId": "ASIAIJJG73VC6IW5OFVQ",
                    "accountId": "112233445566",
                    "arn": "arn:aws:sts::112233445566:assumed-role/ServiceRole/AWSCloudFormation",
                    "invokedBy": "cloudformation.amazonaws.com",
                    "principalId": "AROAJJJJTTTT44445IJJJ:AWSCloudFormation",
                    "sessionContext": {
                        "attributes": {"creationDate": "2020-11-20T06:19:04Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "112233445566",
                            "arn": "arn:aws:iam::112233445566:role/ServiceRole",
                            "principalId": "AROAJJJJTTTT44445IJJJ",
                            "type": "Role",
                            "userName": "ServiceRole",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
                "p_event_time": "2020-11-20 06:19:05.000",
                "p_parse_time": "2020-11-20 06:31:53.258",
                "p_log_type": "AWS.CloudTrail",
                "p_row_id": "ea68a92f0295a6bed49fa8af068faa05",
                "p_any_aws_account_ids": ["112233445566"],
                "p_any_aws_arns": [
                    "arn:aws:ecr:eu-west-1:112233445566:repository/community",
                    "arn:aws:iam::112233445566:role/ServiceRole",
                    "arn:aws:sts::112233445566:assumed-role/ServiceRole/AWSCloudFormation",
                ],
            },
        ),
        RuleTest(
            name="S3 Made Publicly Accessible",
            expected_result=True,
            log={
                "additionalEventData": {
                    "AuthenticationMethod": "AuthHeader",
                    "CipherSuite": "ECDHE-RSA-AES128-SHA",
                    "SignatureVersion": "SigV4",
                    "vpcEndpointId": "vpce-1111",
                },
                "awsRegion": "us-west-2",
                "eventID": "1111",
                "eventName": "PutBucketPolicy",
                "eventSource": "s3.amazonaws.com",
                "eventTime": "2019-01-01T00:00:00Z",
                "eventType": "AwsApiCall",
                "eventVersion": "1.05",
                "recipientAccountId": "123456789012",
                "requestID": "1111",
                "requestParameters": {
                    "bucketName": "example-bucket",
                    "bucketPolicy": {
                        "Statement": [
                            {
                                "Action": "s3:GetBucketAcl",
                                "Effect": "Allow",
                                "Principal": {"AWS": "*"},
                                "Resource": "arn:aws:s3:::example-bucket",
                                "Sid": "Public Access",
                            },
                        ],
                        "Version": "2012-10-17",
                    },
                    "host": ["s3.us-west-2.amazonaws.com"],
                    "policy": [""],
                },
                "responseElements": None,
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
                "userIdentity": {
                    "accessKeyId": "1111",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                    "principalId": "1111",
                    "sessionContext": {
                        "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/example-role",
                            "principalId": "1111",
                            "type": "Role",
                            "userName": "example-role",
                        },
                    },
                    "type": "AssumedRole",
                },
                "vpcEndpointId": "vpce-1111",
            },
        ),
        RuleTest(
            name="S3 Not Made Publicly Accessible",
            expected_result=False,
            log={
                "additionalEventData": {
                    "AuthenticationMethod": "AuthHeader",
                    "CipherSuite": "ECDHE-RSA-AES128-SHA",
                    "SignatureVersion": "SigV4",
                    "vpcEndpointId": "vpce-1111",
                },
                "awsRegion": "us-west-2",
                "eventID": "1111",
                "eventName": "PutBucketPolicy",
                "eventSource": "s3.amazonaws.com",
                "eventTime": "2019-01-01T00:00:00Z",
                "eventType": "AwsApiCall",
                "eventVersion": "1.05",
                "recipientAccountId": "123456789012",
                "requestID": "1111",
                "requestParameters": {
                    "bucketName": "example-bucket",
                    "bucketPolicy": {
                        "Statement": [
                            {
                                "Action": "s3:GetBucketAcl",
                                "Effect": "Allow",
                                "Principal": {"Service": "cloudtrail.amazonaws.com"},
                                "Resource": "arn:aws:s3:::example-bucket",
                                "Sid": "Public Access",
                            },
                        ],
                        "Version": "2012-10-17",
                    },
                    "host": ["s3.us-west-2.amazonaws.com"],
                    "policy": [""],
                },
                "responseElements": None,
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
                "userIdentity": {
                    "accessKeyId": "1111",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                    "principalId": "1111",
                    "sessionContext": {
                        "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/example-role",
                            "principalId": "1111",
                            "type": "Role",
                            "userName": "example-role",
                        },
                    },
                    "type": "AssumedRole",
                },
                "vpcEndpointId": "vpce-1111",
            },
        ),
        RuleTest(
            name="Null Request Parameters",
            expected_result=False,
            log={
                "additionalEventData": {
                    "AuthenticationMethod": "AuthHeader",
                    "CipherSuite": "ECDHE-RSA-AES128-SHA",
                    "SignatureVersion": "SigV4",
                    "vpcEndpointId": "vpce-1111",
                },
                "awsRegion": "us-west-2",
                "eventID": "1111",
                "eventName": "PutBucketPolicy",
                "eventSource": "s3.amazonaws.com",
                "eventTime": "2019-01-01T00:00:00Z",
                "eventType": "AwsApiCall",
                "eventVersion": "1.05",
                "recipientAccountId": "123456789012",
                "requestID": "1111",
                "requestParameters": None,
                "responseElements": None,
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
                "userIdentity": {
                    "accessKeyId": "1111",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                    "principalId": "1111",
                    "sessionContext": {
                        "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/example-role",
                            "principalId": "1111",
                            "type": "Role",
                            "userName": "example-role",
                        },
                    },
                    "type": "AssumedRole",
                },
                "vpcEndpointId": "vpce-1111",
            },
        ),
        RuleTest(
            name="S3 Failed to make Publicly Accessible",
            expected_result=False,
            log={
                "additionalEventData": {
                    "AuthenticationMethod": "AuthHeader",
                    "CipherSuite": "ECDHE-RSA-AES128-SHA",
                    "SignatureVersion": "SigV4",
                    "vpcEndpointId": "vpce-1111",
                },
                "errorCode": "AccessDenied",
                "awsRegion": "us-west-2",
                "eventID": "1111",
                "eventName": "PutBucketPolicy",
                "eventSource": "s3.amazonaws.com",
                "eventTime": "2019-01-01T00:00:00Z",
                "eventType": "AwsApiCall",
                "eventVersion": "1.05",
                "recipientAccountId": "123456789012",
                "requestID": "1111",
                "requestParameters": {
                    "bucketName": "example-bucket",
                    "bucketPolicy": {
                        "Statement": [
                            {
                                "Action": "s3:GetBucketAcl",
                                "Effect": "Allow",
                                "Principal": {"AWS": "*"},
                                "Resource": "arn:aws:s3:::example-bucket",
                                "Sid": "Public Access",
                            },
                        ],
                        "Version": "2012-10-17",
                    },
                    "host": ["s3.us-west-2.amazonaws.com"],
                    "policy": [""],
                },
                "responseElements": None,
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
                "userIdentity": {
                    "accessKeyId": "1111",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                    "principalId": "1111",
                    "sessionContext": {
                        "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/example-role",
                            "principalId": "1111",
                            "type": "Role",
                            "userName": "example-role",
                        },
                    },
                    "type": "AssumedRole",
                },
                "vpcEndpointId": "vpce-1111",
            },
        ),
        RuleTest(
            name="Empty Policy Payload",
            expected_result=False,
            log={
                "additionalEventData": {
                    "AuthenticationMethod": "AuthHeader",
                    "CipherSuite": "ECDHE-RSA-AES128-SHA",
                    "SignatureVersion": "SigV4",
                    "vpcEndpointId": "vpce-1111",
                },
                "awsRegion": "us-west-2",
                "eventID": "1111",
                "eventName": "SetQueueAttributes",
                "eventSource": "s3.amazonaws.com",
                "eventTime": "2019-01-01T00:00:00Z",
                "eventType": "AwsApiCall",
                "eventVersion": "1.05",
                "recipientAccountId": "123456789012",
                "requestID": "1111",
                "requestParameters": {
                    "attributes": {"Policy": ""},
                    "queueUrl": "https://sqs.us-east-1.amazonaws.com/123456789012/example-queue",
                },
                "responseElements": None,
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
                "userIdentity": {
                    "accessKeyId": "1111",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                    "principalId": "1111",
                    "sessionContext": {
                        "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/example-role",
                            "principalId": "1111",
                            "type": "Role",
                            "userName": "example-role",
                        },
                    },
                    "type": "AssumedRole",
                },
                "vpcEndpointId": "vpce-1111",
            },
        ),
        RuleTest(
            name="Invalid JSON Policy (Should Not Alert)",
            expected_result=False,
            log={
                "awsRegion": "us-west-2",
                "eventID": "test-01",
                "eventName": "PutBucketPolicy",
                "eventSource": "s3.amazonaws.com",
                "eventTime": "2024-01-01 00:00:00.000",
                "eventType": "AwsApiCall",
                "requestParameters": {"bucketPolicy": "invalid-json-policy"},
                "userIdentity": {"type": "AssumedRole", "userName": "TestRole"},
            },
        ),
        RuleTest(
            name="Multiple Conditions All Restrictive (Should Not Alert)",
            expected_result=False,
            log={
                "awsRegion": "us-west-2",
                "eventID": "test-02",
                "eventName": "PutResourcePolicy",
                "eventSource": "secretsmanager.amazonaws.com",
                "eventTime": "2024-01-01 00:00:00.000",
                "eventType": "AwsApiCall",
                "requestParameters": {
                    "resourcePolicy": {
                        "Version": "2012-10-17",
                        "Statement": [
                            {
                                "Effect": "Allow",
                                "Principal": "*",
                                "Action": "secretsmanager:GetSecretValue",
                                "Resource": "*",
                                "Condition": {
                                    "StringEquals": {
                                        "aws:PrincipalOrgID": "o-test123456",
                                        "aws:SourceVpc": "vpc-12345678",
                                    },
                                    "IpAddress": {"aws:SourceIp": "10.0.0.0/8"},
                                },
                            },
                        ],
                    },
                },
                "userIdentity": {"type": "AssumedRole", "userName": "TestRole"},
            },
        ),
        RuleTest(
            name="Public Finding Keywords (Should Alert)",
            expected_result=True,
            log={
                "awsRegion": "us-west-2",
                "eventID": "test-03",
                "eventName": "CreateElasticsearchDomain",
                "eventSource": "es.amazonaws.com",
                "eventTime": "2024-01-01 00:00:00.000",
                "eventType": "AwsApiCall",
                "requestParameters": {
                    "accessPolicies": {
                        "Version": "2012-10-17",
                        "Statement": [
                            {
                                "Effect": "Allow",
                                "Principal": "*",
                                "Action": "es:*",
                                "Resource": "*",
                                "Sid": "PublicInternetAccess",
                            },
                        ],
                    },
                },
                "userIdentity": {"type": "AssumedRole", "userName": "TestRole"},
            },
        ),
        RuleTest(
            name="Wildcard Principal No Conditions (Should Alert)",
            expected_result=True,
            log={
                "awsRegion": "us-west-2",
                "eventID": "test-04",
                "eventName": "PutKeyPolicy",
                "eventSource": "kms.amazonaws.com",
                "eventTime": "2024-01-01 00:00:00.000",
                "eventType": "AwsApiCall",
                "requestParameters": {
                    "policy": {
                        "Version": "2012-10-17",
                        "Statement": [{"Effect": "Allow", "Principal": "*", "Action": "kms:Decrypt", "Resource": "*"}],
                    },
                },
                "userIdentity": {"type": "AssumedRole", "userName": "TestRole"},
            },
        ),
        RuleTest(
            name="Mixed Conditions Some Restrictive (Should Alert)",
            expected_result=True,
            log={
                "awsRegion": "us-west-2",
                "eventID": "test-05",
                "eventName": "SetQueueAttributes",
                "eventSource": "sqs.amazonaws.com",
                "eventTime": "2024-01-01 00:00:00.000",
                "eventType": "AwsApiCall",
                "requestParameters": {
                    "attributes": {
                        "Policy": {
                            "Version": "2012-10-17",
                            "Statement": [
                                {
                                    "Effect": "Allow",
                                    "Principal": "*",
                                    "Action": "sqs:*",
                                    "Resource": "*",
                                    "Condition": {"StringEquals": {"aws:PrincipalOrgID": "o-test123456"}},
                                },
                                {"Effect": "Allow", "Principal": "*", "Action": "sqs:SendMessage", "Resource": "*"},
                            ],
                        },
                    },
                },
                "userIdentity": {"type": "AssumedRole", "userName": "TestRole"},
            },
        ),
        RuleTest(
            name="All Restrictive Conditions Types (Should Not Alert)",
            expected_result=False,
            log={
                "awsRegion": "us-west-2",
                "eventID": "test-06",
                "eventName": "PutBucketPolicy",
                "eventSource": "s3.amazonaws.com",
                "eventTime": "2024-01-01 00:00:00.000",
                "eventType": "AwsApiCall",
                "requestParameters": {
                    "bucketPolicy": {
                        "Version": "2012-10-17",
                        "Statement": [
                            {
                                "Effect": "Allow",
                                "Principal": "*",
                                "Action": "s3:GetObject",
                                "Resource": "*",
                                "Condition": {
                                    "StringEquals": {"aws:PrincipalOrgID": "o-test123456"},
                                    "StringLike": {"aws:SourceVpc": "vpc-*"},
                                    "IpAddress": {"aws:SourceIp": ["10.0.0.0/8", "172.16.0.0/12"]},
                                },
                            },
                        ],
                    },
                },
                "userIdentity": {"type": "AssumedRole", "userName": "TestRole"},
            },
        ),
        RuleTest(
            name="Secrets Manager Restricted Access (Should Not Alert)",
            expected_result=False,
            log={
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "eventName": "PutResourcePolicy",
                "eventSource": "secretsmanager.amazonaws.com",
                "eventTime": "2025-03-05 19:48:47.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.11",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "123456789012",
                "requestID": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "requestParameters": {
                    "blockPublicPolicy": True,
                    "resourcePolicy": '{\n  "Version": "2012-10-17",\n  "Statement": [{\n    "Effect": "Allow",\n    "Principal": "*",\n    "Action": "secretsmanager:GetSecretValue",\n    "Resource": "arn:aws:secretsmanager:us-west-2:123456789012:secret:paloma/example-secret-xxxxxx",\n    "Condition": {\n      "StringEquals": {\n        "aws:PrincipalOrgID": "o-xxxxxxxxxx"\n      },\n      "ForAnyValue:StringLike": {\n        "aws:PrincipalArn": ["arn:aws:iam::*:role/ExampleDeploymentRole*", "arn:aws:iam::*:role/ExampleCodeBuild-*"]\n      }\n    }\n  }, \n  {\n    "Effect": "Allow",\n    "Principal": "*",\n    "Action": "secretsmanager:GetSecretValue",\n    "Resource": "arn:aws:secretsmanager:us-west-2:123456789012:secret:paloma/example-secret-xxxxxx",\n    "Condition": {\n      "StringEquals": {\n        "aws:PrincipalOrgID": "o-xxxxxxxxxx"\n      },\n      "ForAnyValue:StringLike": {\n        "aws:PrincipalArn": ["arn:aws:iam::*:role/ExampleDeploymentRole*", "arn:aws:iam::*:role/ExampleCodeBuild-*"]\n      }\n    }\n  },\n  {\n    "Effect": "Allow",\n    "Principal": {\n      "AWS": ["arn:aws:iam::123456789012:role/ExampleRoleAssumption1", "arn:aws:iam::123456789012:role/ExampleRoleAssumption2"]\n    },\n    "Action": ["secretsmanager:Get*", "secretsmanager:Describe*", "secretsmanager:List*"],\n    "Resource": "arn:aws:secretsmanager:us-west-2:123456789012:secret:paloma/example-secret-xxxxxx"\n  }]\n}',
                    "secretId": "arn:aws:secretsmanager:us-west-2:123456789012:secret:paloma/example-secret-xxxxxx",
                },
                "responseElements": {
                    "arn": "arn:aws:secretsmanager:us-west-2:123456789012:secret:paloma/example-secret-xxxxxx",
                    "name": "paloma/example-secret",
                },
                "sessionCredentialFromConsole": True,
                "sourceIPAddress": "10.0.0.1",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "secretsmanager.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/133.0.0.0 Safari/537.36",
                "userIdentity": {
                    "accessKeyId": "EXAMPLEACCESSKEYID",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/AWSReservedSSO_ExampleRole_xxxxxxxxxxxxxxxx/example.user",
                    "principalId": "AROAXXXXXXXXXXXXXXXXX:example.user",
                    "sessionContext": {
                        "attributes": {"creationDate": "2025-03-05T19:41:35Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/aws-reserved/sso.amazonaws.com/AWSReservedSSO_ExampleRole_xxxxxxxxxxxxxxxx",
                            "principalId": "AROAXXXXXXXXXXXXXXXXX",
                            "type": "Role",
                            "userName": "AWSReservedSSO_ExampleRole_xxxxxxxxxxxxxxxx",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="KMS Key Restricted Access (Should Not Alert)",
            expected_result=False,
            log={
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "eventName": "PutKeyPolicy",
                "eventSource": "kms.amazonaws.com",
                "eventTime": "2025-03-05 21:19:44.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.11",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "123456789012",
                "requestID": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "requestParameters": {
                    "bypassPolicyLockoutSafetyCheck": False,
                    "keyId": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "policy": '{\n    "Version": "2008-10-17",\n    "Statement": [\n        {\n            "Effect": "Allow",\n            "Principal": {\n                "AWS": "arn:aws:iam::123456789012:root"\n            },\n            "Action": "kms:*",\n            "Resource": "*"\n        },\n        {\n            "Effect": "Allow",\n            "Principal": {\n                "AWS": [\n                    "arn:aws:iam::123456789012:role/ExampleRoleAssumption1",\n                    "arn:aws:iam::123456789012:role/ExampleRoleAssumption2"\n                ]\n            },\n            "Action": [\n                "kms:Decrypt",\n                "kms:DescribeKey"\n            ],\n            "Resource": "*"\n        },\n        {\n            "Effect": "Allow",\n            "Principal": "*",\n            "Action": "kms:Decrypt",\n            "Resource": "*",\n            "Condition": {\n                "StringEquals": {\n                    "aws:PrincipalOrgID": "o-xxxxxxxxxx"\n                },\n                "ForAnyValue:StringLike": {\n                    "aws:PrincipalArn": [\n                        "arn:aws:iam::*:role/ExampleDeploymentRole*",\n                        "arn:aws:iam::*:role/ExampleCodeBuild-*"\n                    ]\n                }\n            }\n        },\n        {\n            "Effect": "Allow",\n            "Principal": "*",\n            "Action": "kms:Decrypt",\n            "Resource": "*",\n            "Condition": {\n                "StringEquals": {\n                    "aws:PrincipalOrgID": "o-yyyyyyyyyy"\n                },\n                "ForAnyValue:StringLike": {\n                    "aws:PrincipalArn": [\n                        "arn:aws:iam::*:role/ExampleDeploymentRole*",\n                        "arn:aws:sts::*:role/ExampleCodeBuild-*",\n                        "arn:aws:sts::*:assumed-role/ExampleDeploymentRole*",\n                        "arn:aws:sts::*:assumed-role/ExampleCodeBuild-*"\n                    ]\n                }\n            }\n        }\n    ]\n}',
                    "policyName": "default",
                },
                "resources": [
                    {
                        "accountId": "123456789012",
                        "arn": "arn:aws:kms:us-west-2:123456789012:key/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                        "type": "AWS::KMS::Key",
                    },
                ],
                "responseElements": {
                    "keyId": "arn:aws:kms:us-west-2:123456789012:key/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                },
                "sessionCredentialFromConsole": True,
                "sourceIPAddress": "10.0.0.1",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_256_GCM_SHA384",
                    "clientProvidedHostHeader": "kms.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/133.0.0.0 Safari/537.36",
                "userIdentity": {
                    "accessKeyId": "EXAMPLEACCESSKEYID",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/AWSReservedSSO_ExampleRole_xxxxxxxxxxxxxxxx/example.user",
                    "principalId": "AROAXXXXXXXXXXXXXXXXX:example.user",
                    "sessionContext": {
                        "attributes": {"creationDate": "2025-03-05T21:15:00Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/aws-reserved/sso.amazonaws.com/AWSReservedSSO_ExampleRole_xxxxxxxxxxxxxxxx",
                            "principalId": "AROAXXXXXXXXXXXXXXXXX",
                            "type": "Role",
                            "userName": "AWSReservedSSO_ExampleRole_xxxxxxxxxxxxxxxx",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]
